--[[this is the GELua section for retry.
It is called by core/input/actions/raceTools.json to set values for postition, rotation, velocity, enables hasRetrySet, so you can use the retry key binding, and calls vehicle/extensions/autoveResetWSpeed.lua.
It's also called by raceTools.json for retryWithSpeed to use these values, and to call vehicle/extensions/autoveResetWSpeed.lua]]--
local M = {}
M.hasRetrySet = false
local reTryPos
local reTryRot
local velX = 0
local velY = 0
local velZ = 0
local retryHistory = {}
local retryBackStep = 1
local hasLoaded =false

local function onExtensionLoaded()
  print("loaded Race Tool - Retry")
  M.hasRetrySet = false
  M.vehicleTab = {}
  hasLoaded = true
  --be:queueAllObjectLua('extensions.auto_veResetWSpeed.VEclearEntry()')
end

local function GEclearEntry(id)
  --local plId = be:getPlayerVehicleID(0)
  retryHistory[id] = nil
  print("clearing out GE retry table")
end


local function setRetryPosSpeed()
  --print("GELUA Setting Retry with GELua/raceTools")
  local playerVehicle = getPlayerVehicle(0)
  if not playerVehicle then return end --if no vehicle, then "end" this function
  local plId = be:getPlayerVehicleID(0)

  local vX, vY, vZ = playerVehicle:getVelocityXYZ()
  M.vehicleTab[plId] = {
    [plId] = playerVehicle,
    reTryPos = playerVehicle:getPosition(),
    reTryRot = quat(playerVehicle:getClusterRotationSlow(playerVehicle:getRefNodeId())),
    retrySet = true,
    velX = vX,
    velY = vY,
    velZ = vZ
  }

  if not retryHistory[plId] then
    retryHistory[plId] = {}
  end

  table.insert(retryHistory[plId], 1, M.vehicleTab[plId])

  print(plId.."'s amount of retrys set- "..#retryHistory[plId])

  playerVehicle:queueLuaCommand("extensions.auto_veResetWSpeed.veSetRetryValues(" .. #retryHistory[plId] .. ")") 
  --print("Retry location/Rot SET")
  guihooks.trigger('Message', {ttl = 10, msg = 'Retry Location Set', category = "save", icon = "save"})
  M.hasRetrySet = true
end

local lastPress = os.clockhp()
local function retryWithSpeed()
  local tSinceLast = math.abs(os.clockhp() - lastPress)--this should go first in line to not have time pass before checking
  simTimeAuthority.set(0.001)
  local playerVehicleRe = getPlayerVehicle(0)
  local plId = be:getPlayerVehicleID(0)

  if tSinceLast >= 0.01 and tSinceLast <= 0.9 then --this is to check how fast you've pressed the button
    if retryBackStep <= #retryHistory[plId] then --now cycling back, we check if we've reached the end
      retryBackStep = retryBackStep + 1 --if not, we go to next history entry
      print("going to previous set-retry- "..retryBackStep)
    else
      retryBackStep = 1 --if we did reach the end, we start at 1 again
      print("back to first set-retry- "..retryBackStep)
    end
  else
    print("retrying from version- "..retryBackStep) --if we didn't press it quickly enough, it'll be a normal retry
  end

  for k,v in pairs(retryHistory[plId])do
    if k == retryBackStep then
      local cRot = v.reTryRot
      local cPos = v.reTryPos
      playerVehicleRe:setPositionRotation(cPos.x, cPos.y, cPos.z, cRot.x, cRot.y, cRot.z, cRot.w)
      playerVehicleRe:applyClusterVelocityScaleAdd(playerVehicleRe:getRefNodeId(), 1, v.velX, v.velY, v.velZ)
      playerVehicleRe:queueLuaCommand("extensions.auto_veResetWSpeed.veUseRetryValues(".. retryBackStep ..")")
      guihooks.trigger('Message', {ttl = 10, msg = 'Retrying', category = "check_circle", icon = "check_circle"})
    end
  end
  lastPress = os.clockhp() --these should go at the end so the check at the beginning has least amount of time discrepancy
end

local function onUpdate()
  if hasLoaded ==true then
    be:queueAllObjectLua('extensions.auto_veResetWSpeed.VEclearEntry()')
    hasLoaded =false
  end
end

M.onUpdate = onUpdate
M.geTestButton = geTestButton

M.GEclearEntry = GEclearEntry
M.setRetryPosSpeed = setRetryPosSpeed
M.retryWithSpeed = retryWithSpeed
M.onExtensionLoaded = onExtensionLoaded

return M

--[[
  vehDir:set(vehicle:getDirectionVectorXYZ())


  "pSpeed":{
      "cat":"gameplay",
      "order":  94.86,
      "ctx": "tlua",
      "onDown"  :"if extensions.raceTools_geResetWSpeed then extensions.raceTools_geResetWSpeed.setRetryPosSpeed() end",
      "title": "Set Retry Location",
      "desc": "Set location and speed for retrying with speed"
  },
  "reSpeed":{
    "cat":"gameplay",
    "order":  94.87,
    "ctx": "tlua",
    "onDown"  :"if extensions.raceTools_geResetWSpeed and extensions.raceTools_geResetWSpeed.hasRetrySet then extensions.raceTools_geResetWSpeed.retryWithSpeed() end",
    "title": "Retry",
    "desc": "Retry from location with speed"
  }



"geTestButton":{
    "cat":"gameplay",
    "order":  5000,
    "ctx": "tlua",
    "onDown"  :"if extensions.raceTools_geResetWSpeed then extensions.raceTools_geResetWSpeed.geTestButton() end",
    "title": "testing GE button",
    "desc": "experimental GE button"
},
"testButton":{
    "cat":"gameplay",
    "order":  5000,
    "ctx": "vlua",
    "onDown"  :"if extensions.auto_veResetWSpeed then extensions.auto_veResetWSpeed.veTestButton() end",
    "title": "testing button",
    "desc": "experimental button"
},

]]--